import os
import datetime
from urllib3.util import Retry

import requests
from requests.adapters import HTTPAdapter

from em_tasks import logger


class SpreeApi(object):
  def __init__(self, endpoint, api_key, api_version='v1'):
    self.endpoint = endpoint
    self.api_key = api_key
    self.api_version = api_version

    retries = Retry(
      total=7,
      backoff_factor=0.1,
      status_forcelist=[429, 500, 502, 503, 504],
    )
    self.session = requests.Session()
    self.session.mount(self.endpoint, HTTPAdapter(max_retries=retries))

  def list_orders(self, params={}):
    endpoint_url = '{}/api/{}/orders'.format(self.endpoint, self.api_version)
    payload = {
      'token': self.api_key,
      'q[completed_at_not_null]': 1,
      'q[s]': 'completed_at:asc',
      'page': 1,
      'per_page': 50
    }
    payload.update(params)

    # params = '&'.join([k if v is None else '{}={}'.format(k, v) for k, v in payload.items()])
    response = self.session.get(endpoint_url, params=payload)
    logger.debug('[Request] %s', response.request.url)

    return response.json()

  def get_orders(self, order_ids):
    endpoint_url = '{}/api/{}/orders'.format(self.endpoint, self.api_version)
    payload = {
      'token': self.api_key,
      'q[id_in][]': order_ids,
      'q[s]': 'completed_at:asc',
      'page': 1,
      'per_page': len(order_ids)
    }

    response = self.session.get(endpoint_url, params=payload)
    logger.debug('[Request] %s', response.request.url)

    return response.json()

  def approve_order(self, order_number):
    endpoint_url = '{}/api/{}/orders/{}/approve.json'.format(self.endpoint, self.api_version, order_number)
    payload = {'token': self.api_key}
    response = self.session.put(endpoint_url, params=payload)
    logger.debug('[Request] %s', response.request.url)

    return response.json()

  def get_order_refunds(self, order_number):
    endpoint_url = '{}/api/{}/orders/{}/refunds.json'.format(self.endpoint, self.api_version, order_number)
    payload = {'token': self.api_key}
    response = self.session.get(endpoint_url, params=payload)

    return response.json()

  def add_shipment_tracking(self, shipment_id, carrier, tracking):
    endpoint_url = '{}/api/{}/shipments/{}/ship?token={}'.format(self.endpoint, self.api_version, shipment_id, self.api_key)
    payload = {'shipment': {'tracking': tracking, 'carrier': carrier}}

    response = self.session.put(endpoint_url, json=payload)

    return response.json()

  def update_stock(self, stock_item_id, qty, stock_location_id=1, force=True, backorderable=False):
    endpoint_url = '{}/api/{}/stock_locations/{}/stock_items/{}?token={}'.format(self.endpoint, self.api_version, stock_location_id, stock_item_id, self.api_key)
    payload = {'stock_item': {'count_on_hand': qty, 'force': force, 'backorderable': backorderable}}

    return self.session.put(endpoint_url, json=payload)

  def update_variant(self, product_id, variant_id, **params):
    endpoint_url = '{}/api/{}/products/{}/variants/{}?token={}'.format(self.endpoint, self.api_version, product_id, variant_id, self.api_key)
    payload = {'variant': params}

    return self.session.put(endpoint_url, json=payload)

  def download_inventory(self, source, output_path):
    if not output_path:
      return

    output_dir = os.path.dirname(output_path)
    if not os.path.isdir(output_dir):
      os.makedirs(output_dir)

    if os.path.isfile(output_path):
      os.remove(output_path)

    endpoint_url = '{}/api/{}/inventory_reports/download'.format(self.endpoint, self.api_version)
    payload = {'token': self.api_key, 'source': source}

    with self.session.get(endpoint_url, params=payload, stream=True) as r:
      r.raise_for_status()
      with open(output_path, 'wb+') as fh:
        for chunk in r.iter_content(chunk_size=8192):
          fh.write(chunk)

  def get_shop(self):
    endpoint_url = '{}/api/v1/stores'.format(self.endpoint, self.api_version)
    response = self.session.get(endpoint_url, params={'token': self.api_key})

    resp = response.json()
    if 'stores' not in resp or not resp['stores']:
      return

    shop = None
    for store in resp['stores']:
      if store['default']:
        shop = store
        break

    if shop is None:
      shop = resp['stores'][0]

    return shop

  # def get_order(self, order_id):
  #   endpoint_url = self.append_api_key(self.endpoint + '/api/v1/orders/%s' % order_id)

  #   return requests.get(endpoint_url).json()

  # def list_authorized_orders(self, days_back=14):
  #   start_date = datetime.datetime.utcnow() - datetime.timedelta(days=days_back)
  #   start_date = start_date.strftime("%Y-%m-%dT00:00:00")

  #   action = '/api/v1/orders?q[state_eq]=complete&q[payment_state_eq]=balance_due&q[completed_at_gt]=%s&per_page=1000' % start_date
  #   endpoint_url = self.append_api_key(self.endpoint + action)
  #   response = requests.get(endpoint_url)
  #   authorized_orders = response.json()
  #   if 'orders' not in authorized_orders:
  #     return None

  #   return [order for order in authorized_orders['orders'] if order['payments'][-1]['state'] == 'pending']

  # def list_unshipped_orders(self, days_back=14):
  #   start_date = datetime.datetime.utcnow() - datetime.timedelta(days=days_back)
  #   start_date = start_date.strftime("%Y-%m-%dT00:00:00")
  #   action = '/api/v1/orders?q[state_eq]=complete&q[shipment_state_eq]=ready&q[completed_at_gt]=%s&per_page=1000' % start_date
  #   endpoint_url = self.append_api_key(self.endpoint + action)
  #   response = requests.get(endpoint_url)
  #   return response.json()

  # def list_payments(self, order_id):
  #   endpoint_url = self.append_api_key(self.endpoint + '/api/v1/orders/%s/payments' % order_id)
  #   response = requests.get(endpoint_url)
  #   return response.json()

  # def get_authorized_payment_id(self, order_id):
  #   payments = self.list_payments(order_id)
  #   payments = [payment['number'] for payment in payments['payments'] if payment['state'] == 'pending']
  #   if len(payments) > 0:
  #     return payments[0]

  #   return None

  # def capture_payment(self, order_id, payment_id):
  #   endpoint_url = self.append_api_key(self.endpoint + '/api/v1/orders/%s/payments/%s/capture' % (order_id, payment_id))
  #   response = requests.put(endpoint_url)
  #   return response.json()

  # def void_payment(self, order_id):
  #   endpoint_url = self.append_api_key(self.endpoint + '/api/v1/orders/%s/payments/0/void' % order_id)
  #   response = requests.put(endpoint_url)
  #   return response.json()

  # def cancel_order(self, order_id):
  #   endpoint_url = self.append_api_key(self.endpoint + '/api/v1/orders/%s/cancel' % order_id)
  #   response = requests.put(endpoint_url)
  #   return response.json()

  # def ship_order(self, shipment_id, tracking, carrier=''):
  #   carrier_upper = carrier.upper()
  #   if 'UPS' in carrier_upper:
  #       carrier = 'UPS'
  #   if 'USPS' in carrier_upper:
  #       carrier = 'USPS'
  #   if 'FEDEX' in carrier_upper:
  #       carrier = 'FedEx'
  #   if 'DHL' in carrier_upper:
  #       carrier = 'DHL'
  #   if 'AMAZON' in carrier_upper:
  #       carrier = 'Other'
  #   endpoint_url = self.append_api_key(
  #     self.endpoint + '/api/v1/shipments/%s/ship?shipment[tracking]=%s&shipment[carrier]=%s' % (shipment_id, tracking, carrier))
  #   response = requests.put(endpoint_url)
  #   return response.json()

  # def append_api_key(self, url):
  #   separator = '&' if '?' in url else '?'
  #   return url + '%stoken=%s' % (separator, self.api_key)
